/*
  SD card read/write

  This example shows how to read and write data to and from an SD card file
  The circuit:
   SD card attached to SPI bus as follows:
 ** MOSI - pin 11
 ** MISO - pin 12
 ** CLK - pin 13
 ** CS - pin 4 (for MKRZero SD: SDCARD_SS_PIN)

 This device program can translate only string with chars from ['a'..'z' | '0'..'9' | ' ']

*/

#include <SPI.h>
#include <SD.h>

#define SHORT 400

File myFile;
char c;
String str = "sos pm fair";

void long_delay_off() {
  for (int i = 0; i < 9; i++)
    between_char_delay();
}

void long_delay_on() {
  PORTC |= _BV(PORTC0);
  
  for (int i = 0; i < 9; i++)
    between_char_delay();

  PORTC &= ~(_BV(PORTC0));
}

void long_sig() {
  PORTC |= _BV(PORTC0);
  delay(3 * SHORT);
  PORTC &= ~(_BV(PORTC0));
  delay(SHORT);
}

void short_sig() {
  PORTC |= _BV(PORTC0);
  delay(SHORT);
  PORTC &= ~(_BV(PORTC0));
  delay(SHORT);
}

void between_char_delay() {
  delay(4 * SHORT); 
}

void to_morse(char c) {
  switch (c) {
    case 'a':
      // .-
      short_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'b':
      // -...
      long_sig();
      short_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'c':
      // -.-.
      long_sig();
      short_sig();
      long_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'd':
      // -..
      long_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'e':
      // .
      short_sig();
      
      between_char_delay();
      break;

    case 'f':
      // ..-.
      short_sig();
      short_sig();
      long_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'g':
      // --.
      long_sig();
      long_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'h':
      // ....
      short_sig();
      short_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'i':
      // ..
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'j':
      // .---
      short_sig();
      long_sig();
      long_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'k':
      // -.-
      long_sig();
      short_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'l':
      // .-..
      short_sig();
      long_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'm':
      // --
      long_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'n':
      // -.
      long_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'o':
      // ---
      long_sig();
      long_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'p':
      // .--.
      short_sig();
      long_sig();
      long_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 'q':
      // --.-
      long_sig();
      long_sig();
      short_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'r':
      // .-.
      short_sig();
      long_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 's':
      // ...
      short_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case 't':
      // -
      long_sig();
      
      between_char_delay();
      break;

    case 'u':
      // ..-
      short_sig();
      short_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'v':
      // ...-
      short_sig();
      short_sig();
      short_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'w':
      // .--
      short_sig();
      long_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'x':
      // -..-
      long_sig();
      short_sig();
      short_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'y':
      // -.--
      long_sig();
      short_sig();
      long_sig();
      long_sig();
      
      between_char_delay();
      break;

    case 'z':
      // --..
      long_sig();
      long_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;
      
    case '0':
      // -----
      long_sig();
      long_sig();
      long_sig();
      long_sig();
      long_sig();
      
      between_char_delay();
      break;

    case '1':
      // .----
      short_sig();
      long_sig();
      long_sig();
      long_sig();
      long_sig();
      
      between_char_delay();
      break;

    case '2':
      // ..---
      short_sig();
      short_sig();
      long_sig();
      long_sig();
      long_sig();
      
      between_char_delay();
      break;

    case '3':
      // ...--
      short_sig();
      short_sig();
      short_sig();
      long_sig();
      long_sig();
      
      between_char_delay();
      break;

    case '4':
      // ....-
      short_sig();
      short_sig();
      short_sig();
      short_sig();
      long_sig();
      
      between_char_delay();
      break;

    case '5':
      // .....
      short_sig();
      short_sig();
      short_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case '6':
      // -....
      long_sig();
      short_sig();
      short_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case '7':
      // --...
      long_sig();
      long_sig();
      short_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case '8':
      // ---..
      long_sig();
      long_sig();
      long_sig();
      short_sig();
      short_sig();
      
      between_char_delay();
      break;

    case '9':
      // ----.
      long_sig();
      long_sig();
      long_sig();
      long_sig();
      short_sig();
      
      between_char_delay();
      break;

    case ' ':
      // /
      between_char_delay();
      between_char_delay();
      between_char_delay();
      break;

    default:
      // /
      between_char_delay();
      between_char_delay();
      between_char_delay();
      break;
  }
}

void setup() {
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }

  if (!SD.begin(4)) {
    Serial.println("SD initialization failed!");
  } else {
  
    // open the file. note that only one file can be open at a time,
    // so you have to close this one before opening another.
    
    // re-open the file for reading:
    myFile = SD.open("pm.txt");
    if (myFile) {
      // read from the file until there's nothing else in it:
      int i = 0;
      while (myFile.available())
        str[i++] = myFile.read();

      str[i] = '\0';
        
      // close the file:
      myFile.close();
    } else {
      // if the file didn't open, print an error:
      Serial.println("error opening test.txt");
    }
  }

  str.toLowerCase();

  // set LED as output
  DDRC |= _BV(DDC0);

  long_delay_off();
  long_delay_on();
  long_delay_off();
}

void loop() {
  for (int j = 0; j < str.length(); j++) {
      to_morse(str[j]);
      Serial.println(str[j]);
  }

  // delay between content display

  Serial.println("End of content.\nReseting translation...");
  long_delay_off();
  long_delay_on();
  long_delay_off();
}
